Type: Package
Package: pool
Title: Object Pooling
Version: 1.0.5
Authors@R: c(
    person("Joe", "Cheng", , "joe@posit.co", role = "aut"),
    person("Barbara", "Borges", role = "aut"),
    person("Hadley", "Wickham", , "hadley@posit.co", role = c("aut", "cre")),
    person("Posit Software, PBC", role = c("cph", "fnd"))
  )
Description: Enables the creation of object pools, which make it less
    computationally expensive to fetch a new object. Currently the only
    supported pooled objects are 'DBI' connections.
License: MIT + file LICENSE
URL: https://github.com/rstudio/pool, https://rstudio.github.io/pool/
BugReports: https://github.com/rstudio/pool/issues
Depends: methods, R (>= 3.6.0)
Imports: DBI (>= 1.2.1), later (>= 1.0.0), R6, rlang (>= 1.0.0)
Suggests: covr, dbplyr (>= 2.4.0), dplyr, knitr, rmarkdown, RSQLite,
        shiny, testthat (>= 3.0.0), tibble
VignetteBuilder: knitr
Config/Needs/website: tidyverse/tidytemplate
Config/roxygen2/version: 8.0.0
Config/testthat/edition: 3
Encoding: UTF-8
NeedsCompilation: no
Packaged: 2026-05-18 18:53:10 UTC; hadleywickham
Author: Joe Cheng [aut],
  Barbara Borges [aut],
  Hadley Wickham [aut, cre],
  Posit Software, PBC [cph, fnd]
Maintainer: Hadley Wickham <hadley@posit.co>
Repository: CRAN
Date/Publication: 2026-05-19 05:10:02 UTC
Built: R 4.5.2; ; 2026-05-19 10:28:14 UTC; unix
