# Generated by roxygen2: do not edit by hand

export(Pool)
export(dbBreak)
export(dbPool)
export(demoDb)
export(localCheckout)
export(onActivate)
export(onDestroy)
export(onPassivate)
export(onValidate)
export(poolCheckout)
export(poolClose)
export(poolCreate)
export(poolReturn)
export(poolWithTransaction)
exportClasses(Pool)
exportMethods(dbAppendTable)
exportMethods(dbAppendTableArrow)
exportMethods(dbBegin)
exportMethods(dbCommit)
exportMethods(dbCreateTable)
exportMethods(dbCreateTableArrow)
exportMethods(dbDataType)
exportMethods(dbDisconnect)
exportMethods(dbExecute)
exportMethods(dbExistsTable)
exportMethods(dbGetInfo)
exportMethods(dbGetQuery)
exportMethods(dbGetQueryArrow)
exportMethods(dbIsReadOnly)
exportMethods(dbIsValid)
exportMethods(dbListFields)
exportMethods(dbListObjects)
exportMethods(dbListTables)
exportMethods(dbQuoteIdentifier)
exportMethods(dbQuoteLiteral)
exportMethods(dbQuoteString)
exportMethods(dbReadTable)
exportMethods(dbReadTableArrow)
exportMethods(dbRemoveTable)
exportMethods(dbRollback)
exportMethods(dbSendQuery)
exportMethods(dbSendQueryArrow)
exportMethods(dbSendStatement)
exportMethods(dbUnquoteIdentifier)
exportMethods(dbWithTransaction)
exportMethods(dbWriteTable)
exportMethods(dbWriteTableArrow)
exportMethods(onActivate)
exportMethods(onDestroy)
exportMethods(onPassivate)
exportMethods(onValidate)
exportMethods(poolCheckout)
exportMethods(poolClose)
exportMethods(poolReturn)
exportMethods(sqlAppendTable)
exportMethods(sqlCreateTable)
exportMethods(sqlData)
exportMethods(sqlInterpolate)
exportMethods(sqlParseVariables)
import(DBI)
import(methods)
import(rlang)
importFrom(DBI,dbBreak)
importFrom(R6,R6Class)
importFrom(later,later)
importFrom(utils,packageVersion)
