# onValidate() chains underlying error when no query works

    Code
      onValidate(con)
    Condition
      Error in `onValidate()`:
      ! Validation not successful.
      Caused by error:
      ! no such table: another_nonexistent_table

